@@ -49,7 +49,7 @@ except ImportError:
     pep8 = None
 
 
-__version__ = '0.8.3'
+__version__ = '0.8.4'
 
 
 PEP8_BIN = 'pep8'
@@ -58,8 +58,11 @@ LF = '\n'
 CRLF = '\r\n'
 
 
-def open_with_encoding(filename, encoding, mode='r'):
+def open_with_encoding(filename, encoding=None, mode='r'):
     """Return opened file with a specific encoding."""
+    if not encoding:
+        encoding = detect_encoding(filename)
+
     try:
         # Python 3
         return open(filename, mode=mode, encoding=encoding,
@@ -72,36 +75,22 @@ def open_with_encoding(filename, encoding, mode='r'):
 def detect_encoding(filename):
     """Return file encoding."""
     try:
-        # Python 3
-        try:
-            with open(filename, 'rb') as input_file:
-                encoding = tokenize.detect_encoding(input_file.readline)[0]
+        with open(filename, 'rb') as input_file:
+            from lib2to3.pgen2 import tokenize as lib2to3_tokenize
+            encoding = lib2to3_tokenize.detect_encoding(input_file.readline)[0]
 
-                # Check for correctness of encoding
-                import io
-                with io.TextIOWrapper(input_file, encoding) as wrapper:
-                    wrapper.read()
-
-            return encoding
-        except (SyntaxError, LookupError, UnicodeDecodeError):
-            return 'latin-1'
-    except AttributeError:
-        # Python 2
-        encoding = 'utf-8'
-        try:
-            # Check for correctness of encoding
-            with open_with_encoding(filename, encoding) as input_file:
-                input_file.read()
-        except UnicodeDecodeError:
-            encoding = 'latin-1'
+        # Check for correctness of encoding
+        with open_with_encoding(filename, encoding) as test_file:
+            test_file.read()
 
         return encoding
+    except (SyntaxError, LookupError, UnicodeDecodeError):
+        return 'latin-1'
 
 
 def read_from_filename(filename, readlines=False):
     """Return contents of file."""
-    with open_with_encoding(filename,
-                            encoding=detect_encoding(filename)) as input_file:
+    with open_with_encoding(filename) as input_file:
         return input_file.readlines() if readlines else input_file.read()
 
 
@@ -150,7 +139,8 @@ class FixPEP8(object):
         if contents is None:
             self.source = read_from_filename(filename, readlines=True)
         else:
-            self.source = contents.splitlines(True)
+            sio = StringIO(contents)
+            self.source = sio.readlines()
         self.original_source = copy.copy(self.source)
         self.newline = find_newline(self.source)
         self.options = options
@@ -485,7 +475,9 @@ class FixPEP8(object):
         # When multiline strings are involved, pep8 reports the error as
         # being at the start of the multiline string, which doesn't work
         # for us.
-        if '"""' in target or "'''" in target:
+        if ('"""' in target or
+            "'''" in target or
+                target.rstrip().endswith('\\')):
             return []
 
         fixed = fix_whitespace(target,
@@ -574,6 +566,14 @@ class FixPEP8(object):
         target = self.source[line_index]
         offset = result['column'] - 1
 
+        # When multiline strings are involved, pep8 reports the error as
+        # being at the start of the multiline string, which doesn't work
+        # for us.
+        if ('"""' in target or
+            "'''" in target or
+                target.rstrip().endswith('\\')):
+            return []
+
         fixed = fix_whitespace(target,
                                offset=offset,
                                replacement=' ')
@@ -756,7 +756,7 @@ class FixPEP8(object):
         self.source[line_index] = first + self.newline + second
 
     def fix_e711(self, result):
-        """Fix comparison."""
+        """Fix comparison with None."""
         line_index = result['line'] - 1
         target = self.source[line_index]
         offset = result['column'] - 1
@@ -781,6 +781,37 @@ class FixPEP8(object):
 
         self.source[line_index] = ' '.join([left, new_center, right])
 
+    def fix_e712(self, result):
+        """Fix comparison with boolean."""
+        line_index = result['line'] - 1
+        target = self.source[line_index]
+        offset = result['column'] - 1
+
+        right_offset = offset + 2
+        if right_offset >= len(target):
+            return []
+
+        left = target[:offset].rstrip()
+        center = target[offset:right_offset]
+        right = target[right_offset:].lstrip()
+
+        # Handle simple cases only.
+        new_right = None
+        if center.strip() == '==':
+            if re.match(r'\bTrue\b', right):
+                new_right = re.sub(r'\bTrue\b *', '', right, count=1)
+        elif center.strip() == '!=':
+            if re.match(r'\bFalse\b', right):
+                new_right = re.sub(r'\bFalse\b *', '', right, count=1)
+
+        if new_right is None:
+            return []
+
+        if new_right[0].isalnum():
+            new_right = ' ' + new_right
+
+        self.source[line_index] = left + new_right
+
     def fix_e721(self, _):
         """Switch to use isinstance()."""
         return self.refactor('idioms')
@@ -1245,8 +1276,6 @@ class Wrapper(object):
     ])
 
     def __init__(self, physical_lines):
-        if type(physical_lines) != list:
-            physical_lines = physical_lines.splitlines(True)
         self.lines = physical_lines
         self.index = 0
         self.tokens = list()
@@ -1405,7 +1434,8 @@ class Wrapper(object):
 
                 valid_indents[row] = vi
 
-                # ...returning to original continuation_line_identation func...
+                # Returning to original continuation_line_indentation() from
+                # pep8.
                 visual_indent = indent_chances.get(start[1])
                 last_indent = start
                 rel_indent[row] = start[1] - indent_level
@@ -1548,7 +1578,7 @@ def check_syntax(code):
 def filter_results(source, results, aggressive=False):
     """Filter out spurious reports from pep8.
 
-    If aggressive is True, we allow possibly unsafe fixes like E711.
+    If aggressive is True, we allow possibly unsafe fixes (E711, E712).
 
     """
     non_docstring_string_line_numbers = multiline_string_lines(
@@ -1577,7 +1607,7 @@ def filter_results(source, results, aggressive=False):
         if issue_id == 'e101' and '\t' not in split_source[r['line']]:
             continue
 
-        if issue_id == 'e711' and not aggressive:
+        if issue_id in ['e711', 'e712'] and not aggressive:
             continue
 
         # pep8 should not complain about SQLAlchemy queries. SQLAlchemy
@@ -1602,12 +1632,12 @@ def multiline_string_lines(source, include_docstrings=False):
     try:
         for t in tokenize.generate_tokens(sio.readline):
             token_type = t[0]
-            token_string = t[1]
+            start_row = t[2][0]
+            end_row = t[3][0]
             start_row = t[2][0]
             end_row = t[3][0]
 
-            if (token_type == tokenize.STRING and
-                    starts_with_triple(token_string)):
+            if (token_type == tokenize.STRING and start_row != end_row):
                 if (include_docstrings or
                         previous_token_type != tokenize.INDENT):
                     # We increment by one since we want the contents of the
@@ -1621,12 +1651,6 @@ def multiline_string_lines(source, include_docstrings=False):
     return line_numbers
 
 
-def starts_with_triple(string):
-    """Return True if the string starts with triple single/double quotes."""
-    return (string.strip().startswith('"""') or
-            string.strip().startswith("'''"))
-
-
 def shorten_comment(line, newline, max_line_length):
     """Return trimmed or split long comment line."""
     assert len(line) > max_line_length
@@ -1660,10 +1684,11 @@ def format_block_comments(source):
         # Optimization.
         return source
 
-    string_line_numbers = multiline_string_lines(source)
+    string_line_numbers = multiline_string_lines(source,
+                                                 include_docstrings=True)
     fixed_lines = []
-    for (line_number, line) in enumerate(source.splitlines(True),
-                                         start=1):
+    sio = StringIO(source)
+    for (line_number, line) in enumerate(sio.readlines(), start=1):
         if (re.match(r'\s*#+\w+', line) and
                 line_number not in string_line_numbers):
             fixed_lines.append(_get_indentation(line) +
@@ -1675,38 +1700,32 @@ def format_block_comments(source):
     return ''.join(fixed_lines)
 
 
-def normalize_line_endings(text):
+def normalize_line_endings(lines):
     """Return fixed line endings.
 
     All lines will be modified to use the most common line ending.
 
     """
-    split_lines = text.splitlines(True)
-    return find_newline(split_lines).join(
-        [line.rstrip('\n\r') for line in split_lines] +
-        [''])
+    newline = find_newline(lines)
+    return [line.rstrip('\n\r') + newline for line in lines]
 
 
 def fix_file(filename, opts=None, output=None):
     if not opts:
         opts = parse_args([filename])[0]
 
-    tmp_source = normalize_line_endings(read_from_filename(filename))
+    original_source = read_from_filename(filename, readlines=True)
 
-    # Add missing newline (important for diff)
-    if tmp_source:
-        tmp_newline = find_newline(tmp_source.splitlines(True))
-        if tmp_source == tmp_source.rstrip(tmp_newline):
-            tmp_source += tmp_newline
+    tmp_source = ''.join(normalize_line_endings(original_source))
 
     fix = FixPEP8(filename, opts, contents=tmp_source)
     fixed_source = fix.fix()
-    original_source = copy.copy(fix.original_source)
     tmp_filename = filename
     if not pep8 or opts.in_place:
         encoding = detect_encoding(filename)
 
-    fixed_source = format_block_comments(fixed_source)
+    if not opts.minimal:
+        fixed_source = format_block_comments(fixed_source)
 
     interruption = None
     try:
@@ -1793,7 +1812,10 @@ def parse_args(args):
                       help='set maximum allowed line length '
                            '(default: %default)')
     parser.add_option('--aggressive', action='store_true',
-                      help='enable possibly unsafe changes (E711)')
+                      help='enable possibly unsafe changes (E711, E712)')
+    parser.add_option('--minimal', action='store_true',
+                      help='only fix issues reported by the pep8 utility; '
+                           'do not fix block comments')
     opts, args = parser.parse_args(args)
 
     if not len(args) and not opts.list_fixes:
