@@ -1,8 +1,8 @@
+#!/usr/bin/env python
 # coding: utf-8
+
 import os
 import sys
-import codecs
-import locale
 
 if sys.version_info < (2, 7):
     import unittest2 as unittest
@@ -61,6 +61,12 @@ class TestUtils(unittest.TestCase):
             autopep8.detect_encoding(
                 os.path.join(ROOT_DIR, 'setup.py')))
 
+    def test_detect_encoding_with_cookie(self):
+        self.assertEqual(
+            'iso-8859-1',
+            autopep8.detect_encoding(
+                os.path.join(ROOT_DIR, 'test', 'iso_8859_1.py')))
+
     def test_read_from_filename_with_bad_encoding(self):
         """Bad encoding should not cause an exception."""
         self.assertEqual(
@@ -70,8 +76,11 @@ class TestUtils(unittest.TestCase):
 
     def test_read_from_filename_with_bad_encoding2(self):
         """Bad encoding should not cause an exception."""
-        self.assertTrue(autopep8.read_from_filename(
-            os.path.join(ROOT_DIR, 'test', 'bad_encoding2.py')))
+        # This causes a warning on Python 3.
+        import warnings
+        with warnings.catch_warnings(record=True):
+            self.assertTrue(autopep8.read_from_filename(
+                os.path.join(ROOT_DIR, 'test', 'bad_encoding2.py')))
 
     def test_fix_whitespace(self):
         self.assertEqual(
@@ -230,15 +239,15 @@ def foo():
 
     def test_normalize_line_endings(self):
         self.assertEqual(
-            'abc\ndef\n123\nhello\nworld\n',
+            ['abc\n', 'def\n', '123\n', 'hello\n', 'world\n'],
             autopep8.normalize_line_endings(
-                'abc\ndef\n123\rhello\r\nworld\r'))
+                ['abc\n', 'def\n', '123\r', 'hello\r\n', 'world\r']))
 
     def test_normalize_line_endings_with_crlf(self):
         self.assertEqual(
-            'abc\r\ndef\r\n123\r\nhello\r\nworld\r\n',
+            ['abc\r\n', 'def\r\n', '123\r\n', 'hello\r\n', 'world\r\n'],
             autopep8.normalize_line_endings(
-                'abc\ndef\r\n123\r\nhello\r\nworld\r'))
+                ['abc\n', 'def\r\n', '123\r\n', 'hello\r\n', 'world\r']))
 
 
 class TestFixPEP8Error(unittest.TestCase):
@@ -890,6 +899,16 @@ a
 b
 c
 ''' )
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
+    def test_e202_skip_multiline_with_escaped_newline(self):
+        """We skip this since pep8 reports the error as being on line 1."""
+        line = r"""
+
+('c\
+' )
 """
         self._inner_setup(line)
         self.assertEqual(self.result, line)
@@ -930,6 +949,28 @@ c
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e221_should_skip_multiline(self):
+        line = '''
+    def javascript(self):
+        return u"""
+<script type="text/javascript" src="++resource++ptg.shufflegallery/jquery.promptu-menu.js"></script>
+<script type="text/javascript">
+$(function(){
+    $('ul.promptu-menu').promptumenu({width: %(width)i, height: %(height)i, rows: %(rows)i, columns: %(columns)i, direction: '%(direction)s', intertia: %(inertia)i, pages: %(pages)i});
+\t$('ul.promptu-menu a').click(function(e) {
+        e.preventDefault();
+    });
+    $('ul.promptu-menu a').dblclick(function(e) {
+        window.location.replace($(this).attr("href"));
+    });
+});
+</script>
+        """  % {
+        }
+'''.lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
     def test_e222(self):
         line = "a = 1 +  1\n"
         fixed = "a = 1 + 1\n"
@@ -1531,6 +1572,18 @@ class Useless(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_with_semicolon_at_end(self):
+        line = "print 1;\n"
+        fixed = "print 1\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e702_with_semicolon_and_space_at_end(self):
+        line = "print 1; \n"
+        fixed = "print 1\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e702_with_whitespace(self):
         line = "print 1 ; print 2\n"
         fixed = "print 1\nprint 2\n"
@@ -1685,6 +1738,35 @@ raise IOError('abc '
         self._inner_setup(line, options=['--aggressive'])
         self.assertEqual(self.result, line)
 
+    def test_e712(self):
+        line = 'foo == True\n'
+        fixed = 'foo\n'
+        self._inner_setup(line, options=['--aggressive', '--select=E712'])
+        self.assertEqual(self.result, fixed)
+
+    def test_e712_in_conditional_with_multiple_instances(self):
+        line = 'if foo == True and bar == True:\npass\n'
+        fixed = 'if foo and bar:\npass\n'
+        self._inner_setup(line, options=['--aggressive', '--select=E712'])
+        self.assertEqual(self.result, fixed)
+
+    def test_e712_with_false(self):
+        line = 'foo != False\n'
+        fixed = 'foo\n'
+        self._inner_setup(line, options=['--aggressive', '--select=E712'])
+        self.assertEqual(self.result, fixed)
+
+    def test_e712_only_if_aggressive(self):
+        line = 'foo == True\n'
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
+    def test_e711_and_e712(self):
+        line = 'if (foo == None and bar == True) or (foo != False and bar != None):\npass\n'
+        fixed = 'if (foo is None and bar) or (foo and bar is not None):\npass\n'
+        self._inner_setup(line, options=['--aggressive'])
+        self.assertEqual(self.result, fixed)
+
     def test_e721(self):
         line = "type('') == type('')\n"
         fixed = "isinstance('', type(''))\n"
@@ -1703,6 +1785,19 @@ raise IOError('abc '
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    @unittest.skipIf(sys.version_info[0] < 3,
+                     'On Python 2 the vertical tab gets turned into a line feed')
+    def test_should_preserve_vertical_tab(self):
+        line = """
+#Memory Bu\vffer Register:
+""".lstrip()
+        fixed = """
+# Memory Bu\vffer Register:
+""".lstrip()
+
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
 
 class TestFixPEP8Warning(unittest.TestCase):
 
@@ -2092,7 +2187,7 @@ class TestOptions(unittest.TestCase):
         self._inner_setup(line, ['--diff'])
         self.assertEqual('\n'.join(self.result.split('\n')[3:]), fixed)
 
-    def test_diff_wtih_empty_file(self):
+    def test_diff_with_empty_file(self):
         self._inner_setup('', ['--diff'])
         self.assertEqual('\n'.join(self.result.split('\n')[3:]), '')
 
